@@ -25,7 +25,7 @@ use Symbol;
 
 # NOTE: due to CPAN version checks this cannot currently be changed to a
 # standard version string, i.e. '0.21'
-our $VERSION = '40';
+our $VERSION = '43';
 
 # this is going to be rewritten to use AUTOLOAD + method caching in a future version
 BEGIN {
@@ -457,32 +457,29 @@ sub update_v3 ($) {
     open( $log_fh, "<$self->{LOGFILE}" )
         || croak "could not open file $self->{LOGFILE} for reading: $!";
 
-# change the first line of the RE to this:
-# (info|programstatus|hoststatus|servicestatus|contactstatus|servicecomment|hostcomment|servicedowntime|hostdowntime) \s* {(
-# to make it a bit more careful, but it has a measurable cost on runtime
-    my $entry_re = qr/
-        # capture the type into $1
-        (\w+) \s*
-        # capture all of the text between the brackets into $2
-        {( .*? )}
-        # match the last bracket only if followed by another definition
-        (?=(?: \s* (?:info|programstatus|hoststatus|servicestatus|contacstatus|servicecomment|hostcomment|servicedowntime|hostdowntime) \s* { | \Z) )
-        # capture remaining text (1-2 lines) into $3 for re-processing
-        (.*)$
-    /xs;
-
-    my @lines = <$log_fh>;
-    my $file  = "@lines";
-
-#Drop comments if we don't need them as it should speed things up a little bit.
-#Comment out the line below if you do want to keep comments
-    $file =~ s/#.*\n//mg;
-    $file =~ s/[\r\n]+\s*/\n/g;    # clean up whitespace and newlines
-
-    while ( $file =~ /$entry_re/g ) {
-        ( my $type, my $text ) = ( $1, $2 );
-        my %item = map { split /\s*=\s*/, $_, 2 } split /\n/, $text;
-        $handlers{$type}->( \%item );
+    my %valid_types = map { ( $_ => 1 ) } qw(info programstatus hoststatus servicestatus contactstatus servicecomment hostcomment servicedowntime hostdowntime);
+    my $entry = '';
+    my %attributes;
+    my $type = 0;
+    while ( my $line = <$log_fh> ) {
+        next if ( $line =~ /^\s*#|^\s*$/ );
+        if ( $line =~ /^\s*(\w+)\s*{\s*$/ ) {
+            %attributes = ();
+            if (exists $valid_types{$1}) {
+                $type = $1;
+            } else {
+                $type = 0;
+            }
+        }
+        if ( $line =~ /^\s*}\s*$/ ) {
+            # Only save the object if it is a valid type
+            if ($type) {
+                $handlers{$type}->( \%attributes );
+            }
+        }
+        if ( $line =~ /\s*(\w+)=(.*)$/ ) {
+            $attributes{$1} = $2;
+        }
     }
 
     close($log_fh);
@@ -540,16 +537,16 @@ Nagios::Service::Status has the following accessor methods (For V1):
 sub service {
     my ( $self, $host, $service ) = @_;
 
-    if ( ref $host eq 'Nagios::Host' ) {
+    if ( ref($host) eq 'Nagios::Host' ) {
         $host = $host->host_name;
     }
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
-    if ( ref $service eq 'Nagios::Service' ) {
+    if ( ref($service) eq 'Nagios::Service' ) {
         $service = $service->service_description;
     }
 
@@ -593,7 +590,7 @@ Returns an array of services descriptions for a given host.
 
 sub list_services_on_host {
     my ( $self, $host ) = @_;
-    if ( ref $host && UNIVERSAL::can( $host, 'host_name' ) ) {
+    if ( ref($host) && UNIVERSAL::can( $host, 'host_name' ) ) {
         $host = $host->host_name;
     }
     return keys %{ $self->{SERVICE}{$host} };
@@ -634,7 +631,7 @@ Nagios::Host::Status has the following accessor methods (for V1):
 sub host {
     my ( $self, $host ) = @_;
 
-    if ( ref $host =~ /^Nagios::(Host|Service)$/ ) {
+    if ( ref($host) =~ /^Nagios::(Host|Service)$/ ) {
         $host = $host->host_name;
     }
 
@@ -696,7 +693,7 @@ Nagios::Contact::Status has the following accessor methods (for v3):
 sub contact {
     my ( $self, $name ) = @_;
 
-    $name = $name->contact_name if ( ref $name eq 'Nagios::Contact' );
+    $name = $name->contact_name if ( ref($name) eq 'Nagios::Contact' );
 
     return undef if ( !$self->{CONTACT}{$name} );
 
@@ -732,7 +729,7 @@ Nagios::Hostcomment::Status is a perl HASH, keyed with the Nagios comment IDs, w
 sub hostcomment {
     my ( $self, $host ) = @_;
 
-    $host = $host->host_name if ( ref $host =~ /^Nagios::(Host|Service)$/ );
+    $host = $host->host_name if ( ref($host) =~ /^Nagios::(Host|Service)$/ );
 
     return undef if ( !$self->{HOSTCOMMENT}{$host} );
     foreach my $id ( keys %{ $self->{HOSTCOMMENT}{$host} } ) {
@@ -773,17 +770,17 @@ Nagios::Servicecomment::Status is a perl HASH, keyed with the Nagios comment IDs
 sub servicecomment {
     my ( $self, $host, $service ) = @_;
 
-    $host = $host->host_name if ( ref $host eq 'Nagios::Host' );
+    $host = $host->host_name if ( ref($host) eq 'Nagios::Host' );
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
     $service = $service->service_description
-        if ( ref $service eq 'Nagios::Service' );
+        if ( ref($service) eq 'Nagios::Service' );
 
-    return undef if ( !$self->{SERVICECOMMENT}{$host}{$service} );
+    return undef if ( !$host || !$service || !$self->{SERVICECOMMENT}{$host}{$service} );
     foreach my $id ( keys %{ $self->{SERVICECOMMENT}{$host}{$service} } ) {
         bless( $self->{SERVICECOMMENT}{$host}{$service}{$id},
             'Nagios::Servicecomment::Status' );
@@ -821,7 +818,7 @@ Nagios::Hostdowntime::Status is a perl HASH, keyed with the Nagios downtime IDs,
 sub hostdowntime {
     my ( $self, $host ) = @_;
 
-    $host = $host->host_name if ( ref $host =~ /^Nagios::(Host|Service)$/ );
+    $host = $host->host_name if ( ref($host) =~ /^Nagios::(Host|Service)$/ );
 
     return undef if ( !$self->{HOSTDOWNTIME}{$host} );
     foreach my $id ( keys %{ $self->{HOSTDOWNTIME}{$host} } ) {
@@ -865,15 +862,15 @@ Nagios::Servicedowntime::Status is a perl HASH, keyed with the Nagios downtime I
 sub servicedowntime {
     my ( $self, $host, $service ) = @_;
 
-    $host = $host->host_name if ( ref $host eq 'Nagios::Host' );
+    $host = $host->host_name if ( ref($host) eq 'Nagios::Host' );
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
     $service = $service->service_description
-        if ( ref $service eq 'Nagios::Service' );
+        if ( ref($service) eq 'Nagios::Service' );
 
     return undef if ( !$self->{SERVICEDOWNTIME}{$host}{$service} );
     foreach my $id ( keys %{ $self->{SERVICEDOWNTIME}{$host}{$service} } ) {
